@@ -1,15 +1,17 @@
 import operator
 from dataclasses import dataclass, field
-from typing import Dict, Tuple, Callable
+from typing import Dict, List, Tuple, Callable
 
 import pandas as pd
 from sympy import Indexed
 
 from finstmt import BalanceSheets, IncomeStatements
 from finstmt.config_manage.statements import StatementsConfigManager
+from finstmt.exc import MismatchingDatesException
 from finstmt.findata.statementsbase import FinStatementsBase
 from finstmt.forecast.config import ForecastConfig
 from finstmt.forecast.main import Forecast
+from finstmt.items.config import ItemConfig
 
 
 @dataclass
@@ -28,9 +30,22 @@ class FinancialStatements:
     """
     income_statements: IncomeStatements
     balance_sheets: BalanceSheets
+    calculate: bool = True
 
 
     def __post_init__(self):
+        from finstmt.resolver.history import StatementsResolver
+
+        self._create_config_from_statements()
+
+        if self.calculate:
+            resolver = StatementsResolver(self)
+            new_stmts = resolver.to_statements()
+            self.income_statements = new_stmts.income_statements
+            self.balance_sheets = new_stmts.balance_sheets
+            self._create_config_from_statements()
+
+    def _create_config_from_statements(self):
         config_dict = {}
         config_dict['income_statements'] = self.income_statements.config
         config_dict['balance_sheets'] = self.balance_sheets.config
@@ -96,7 +111,9 @@ class FinancialStatements:
             'non_cash_expenses',
             'fcf',
             'forecast',
+            'forecasts',
             'forecast_assumptions',
+            'dates',
         ]
         all_config = self.income_statements.config.items + self.balance_sheets.config.items
         item_attrs = [config.key for config in all_config]
@@ -116,115 +133,36 @@ class FinancialStatements:
         return self.net_income + self.non_cash_expenses - self.change('nwc') - self.capex
 
     def forecast(self, **kwargs) -> 'FinancialStatements':
-        from finstmt.forecast.statements import ForecastedFinancialStatements
+        from finstmt.resolver.forecast import ForecastResolver
+
+        if 'bs_diff_max' in kwargs:
+            bs_diff_max = kwargs['bs_diff_max']
+        else:
+            bs_diff_max = ForecastConfig.bs_diff_max
+
+        if 'balance' in kwargs:
+            balance = kwargs['balance']
+        else:
+            balance = ForecastConfig.balance
+
+        self._validate_dates()
 
-        # TODO [#6]: clean up forecast logic
-        #
-        # Seems like this whole thing is repeating a lot of logic that could maybe be removed if
-        # I could construct partial financial statements.
-        #
-        # Restructuring the sympy variables integration into its own library could help.
-        #
-        # This code also feels very messy in general.
         all_forecast_dict = {}
         all_results = {}
-        all_pct_results = {}
         for stmt in [self.income_statements, self.balance_sheets]:
-            forecast_dict, results, pct_results = stmt._forecast(self, **kwargs)
+            forecast_dict, results = stmt._forecast(self, **kwargs)
             all_forecast_dict.update(forecast_dict)
             all_results.update(results)
-            all_pct_results.update(pct_results)
 
-        # Set up for creating dictionary for sympy substitutions. First extract all the results into a dict of dicts
-        # nested dict where keys are date indices, values are dicts where keys are item keys, values are item values
-        by_date_item_dict: Dict[pd.Timestamp, Dict[str, float]] = {}
-        all_dates = list(all_results.values())[0].index.tolist()
+        resolver = ForecastResolver(self, all_forecast_dict, all_results, bs_diff_max, balance=balance)
+        obj = resolver.to_statements()
 
-        def add_series_to_by_date_item_dict(series: pd.Series, item_key: str):
-            for date, value in series.iteritems():
-                date_idx = all_dates.index(date)
-                if date_idx not in by_date_item_dict:
-                    by_date_item_dict[date_idx] = {}
-                by_date_item_dict[date_idx][item_key] = value
-
-        [add_series_to_by_date_item_dict(result_series, item_key) for item_key, result_series in all_results.items()]
-        t = self.config.sympy_namespace['t']
-
-        def get_subs_dict(t_offset: int):
-            subs_dict = {}
-            # TODO [#7]: in forecast calculation process, need to grab previous values
-            #
-            # Currently only getting current period values.
-            values_dict = by_date_item_dict[t_offset]
-            for item_key, item_symbol in self.config.sympy_namespace.items():
-                if item_key in values_dict:
-                    indexed_symbol = item_symbol.__getitem__(t)  # eg cash[t] or cash[t-1]
-                    subs_dict[indexed_symbol] = values_dict[item_key]
-            return subs_dict
-
-        def get_expr_eval_create_series_and_add_to_by_date_item_dict_and_results(overall_item_key: str) -> pd.Series:
-            result_dict = {}
-            expr = self.config.expr_for(overall_item_key)
-            for i, date in enumerate(all_dates):
-                # Now create the subs dict from the dict of dicts
-                subs_dict = get_subs_dict(i)
-                substituted = expr.subs(subs_dict)
-                try:
-                    eval_expr = float(substituted)
-                except TypeError:
-                    # This means wasn't completely substituted. Must be a calculated item which has not yet
-                    # been calculated. Calculate it now.
-                    reverse_sympy_ns = {value: key for key, value in self.config.sympy_namespace.items()}
-                    for sym in substituted.free_symbols:
-                        if isinstance(sym, Indexed):
-                            # Got an uncalculated symbol
-                            uncalc_key = reverse_sympy_ns[sym.base]
-                            get_expr_eval_create_series_and_add_to_by_date_item_dict_and_results(uncalc_key)
-                    # Now try the original calculation again, now that missing items have been calculated
-                    subs_dict = get_subs_dict(i)
-                    substituted = expr.subs(subs_dict)
-                    eval_expr = float(substituted)
-                result_dict[date] = eval_expr
-            result_series = pd.Series(result_dict)
-            result_series.name = self.config.get_value(overall_item_key, 'primary_name')
-            # Add this newly calculated series to overall substitution values
-            add_series_to_by_date_item_dict(result_series, overall_item_key)
-            all_results[overall_item_key] = result_series
-            return result_series
-
-        # Resolve pct of items
-        for pct_item_key, pct_result in all_pct_results.items():
-            item_config = self.config.get(pct_item_key)
-            pct_of_key = item_config.forecast_config.pct_of
-            pct_of_config, pct_of_source_key = self.config._get(pct_of_key)
-            if pct_of_key in all_results:
-                # Already have result, just get it from results
-                pct_of_series = all_results[item_config.forecast_config.pct_of]
-            elif pct_of_config.expr_str is not None:
-                # This is a calculated item, need to calculate it, add it to results, and use that series
-                pct_of_series = get_expr_eval_create_series_and_add_to_by_date_item_dict_and_results(pct_of_key)
-            else:
-                raise ValueError(f'could not get series for {pct_of_key}')
-            calc_series = pct_result * pct_of_series
-            calc_series.name = pct_result.name
-            all_results[pct_item_key] = calc_series
-            # Add this newly calculated series to overall substitution values
-            add_series_to_by_date_item_dict(calc_series, pct_item_key)
-
-        all_results = pd.concat(list(all_results.values()), axis=1).T
-        inc_df = self.income_statements.__class__.from_df(all_results, items_config=self.income_statements.config.items)
-        bs_df = self.balance_sheets.__class__.from_df(all_results, items_config=self.balance_sheets.config.items)
-
-        # type ignore added because for some reason mypy is not picking up structure
-        # correctly since it is a dataclass
-        obj = ForecastedFinancialStatements(inc_df, bs_df, all_forecast_dict)  # type: ignore
         return obj
 
     @property
     def forecast_assumptions(self) -> pd.DataFrame:
-        all_configs = self.income_statements.config.items + self.balance_sheets.config.items  # type: ignore
         all_series = []
-        for config in all_configs:
+        for config in self.all_config_items:
             if config.extract_names is None or not config.forecast_config.make_forecast:
                 continue
             config_series = config.forecast_config.to_series()
@@ -232,6 +170,28 @@ class FinancialStatements:
             all_series.append(config_series)
         return pd.concat(all_series, axis=1).T
 
+    @property
+    def all_config_items(self) -> List[ItemConfig]:
+        return self.income_statements.config.items + self.balance_sheets.config.items  # type: ignore
+
+    @property
+    def dates(self) -> List[pd.Timestamp]:
+        self._validate_dates()
+        return list(self.balance_sheets.statements.keys())
+
+    def _validate_dates(self):
+        bs_dates = set(self.balance_sheets.statements.keys())
+        is_dates = set(self.income_statements.statements.keys())
+        if bs_dates != is_dates:
+            bs_unique = bs_dates.difference(is_dates)
+            is_unique = is_dates.difference(bs_dates)
+            message = 'Got mismatching dates between historical statements. '
+            if bs_unique:
+                message += f'Balance sheet has {bs_unique} dates not in Income Statement. '
+            if is_unique:
+                message += f'Income Statement has {is_unique} dates not in Balance Sheet. '
+            raise MismatchingDatesException(message)
+
     def __add__(self, other):
         statements = _combine_statements(self, other, operator.add)
         return _new_statements(self, other, *statements)
@@ -258,7 +218,7 @@ class FinancialStatements:
         return _new_statements(self, other, *statements)
 
     def __rtruediv__(self, other):
-        # TODO: implement right division for statements
+        # TODO [#41]: implement right division for statements
         raise NotImplementedError(f'cannot divide type {type(other)} by type {type(self)}')
 
 
